#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/resource.h>

#include "srv.h"

bool restart = 0;

int main (int argc, char *argv[])
{
	struct service service;
	char *action;
	int i;
	pid_t pid;
	int retstatus;

	if (argc != 3) {
		fprintf(stderr, "usage: service servicename action\n");
		exit(1);
	}

	if (service_read(argv[1], &service) < 0) {
		fprintf(stderr, "invalid service\n");
		exit(1);
	}

	if (service.script == NULL || service.script[0] == '\0') {
		fprintf(stderr, "no script defined\n");
		exit(1);
	}
	if (service.script[0] != '/') {
		fprintf(stderr, "script needs an absolute pathname\n");
		exit(1);
	}

	action = argv[2];
	if (!strcmp(action, "start") || !strcmp(action, "stop") || !strcmp(action, "status")) {
		/* nothing */
	} else if (!strcmp(action, "restart")) {
		restart = 1;
	} else if (!strcmp(action, "reload") && service.supportsreload) {
		/* nothing */
	} else if (!strcmp(action, "force-reload")) {
		if (service.supportsreload) {
			action = "reload";
		} else {
			restart = 1;
		}
	} else {
		fprintf(stderr, "Invalid action. Valid actions are:\n");
		fprintf(stderr, "start stop restart %sforce-reload status\n",
		  service.supportsreload ? "reload " : "");
		exit(1);
	}

	setpriority(PRIO_PROCESS, 0, service.nice);

	setenv("RUNNING_VIA_SERVICE", "1", 1);

	if (restart) {
		pid = fork();
		if (pid < 0) {
			fprintf(stderr, "fork failed()\n");
			exit(1);
		}
		if (pid == 0) {
			execl(service.script, service.script, "stop", NULL);
			fprintf(stderr, "execl failed\n");
			exit(1);
		}
		while (1) {
			i = waitpid(pid, &retstatus, 0);
			if (i < 0) {
				fprintf(stderr, "waitpid failed\n");
				exit(1);
			}
			if (i == pid) break;
		}
		if (!WIFEXITED(retstatus)) exit(1);
		if (WEXITSTATUS(retstatus) != 0) exit(WEXITSTATUS(retstatus));
		action = "start";
	}

	execl(service.script, service.script, action, NULL);
	fprintf(stderr, "execl failed\n");
	exit(1);
}
