#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>

#include "srv.h"

char *usage =
"Usage:\n"
"  srvmgr --list\n"
"  srvmgr --show servicename\n"
"  srvmgr --show-default servicename\n"
"  srvmgr --show-custom servicename\n"
"  srvmgr --set servicename keyword value\n"
"  srvmgr --clear servicename keyword";

int main (int argc, char *argv[])
{
	char buf[PATH_MAX];
	FILE *f;
	struct service service;
	struct services services;

	if (argc < 2) {
		message(usage);
		exit(1);
	}

	if (!strcmp(argv[1], "--list")) {
		if (argc != 2) {
			message("--list doesn't take an argument");
			exit(1);
		}
		if (services_read_list(&services) < 0) exit(1);
		if (services_dump_list(&services, stdout) < 0) exit(1);
		return 0;
	}

	if (!strcmp(argv[1], "--show")) {
		if (argc != 3) {
			message("--show requires one argument");
			exit(1);
		}
		use_current_language();
		if (service_read(argv[2], &service) < 0) exit(1);
		if (service_dump(&service, stdout) < 0) exit(1);
		return 0;
	}

	if (!strcmp(argv[1], "--show-default")) {
		if (argc != 3) {
			message("--show-default requires one argument");
			exit(1);
		}
		use_current_language();
		if (service_read_default(argv[2], &service) < 0) exit(1);
		if (service_dump(&service, stdout) < 0) exit(1);
		return 0;
	}

	if (!strcmp(argv[1], "--show-custom")) {
		if (argc != 3) {
			message("--show-custom requires one argument");
			exit(1);
		}
		use_current_language();
		if (service_read_custom(argv[2], &service) < 0) exit(1);
		if (service_dump(&service, stdout) < 0) exit(1);
		return 0;
	}

	if (!strcmp(argv[1], "--set")) {
		if (argc != 5) {
			message("--set requires three arguments");
			exit(1);
		}
		if (service_read_custom(argv[2], &service) < 0) exit(1);
		if (service_add2(argv[3], argv[4], &service) < 0) exit(1);
		sprintf(buf, CUSTOMDIR "/%s.service", argv[2]);
		if ((f = fopen(buf, "w")) == NULL) {
			message_e("couldn't open output file");
			exit(1);
		}
		if (service_dump(&service, f) < 0) exit(1);
		fclose(f);
		return 0;
	}

	if (!strcmp(argv[1], "--clear")) {
		if (argc != 4) {
			message("--clear requires two arguments");
			exit(1);
		}
		if (service_read_custom(argv[2], &service) < 0) exit(1);
		if (service_remove1(argv[3], &service) < 0) exit(1);
		sprintf(buf, CUSTOMDIR "/%s.service", argv[2]);
		if ((f = fopen(buf, "w")) == NULL) {
			message_e("couldn't open output file");
			exit(1);
		}
		if (service_dump(&service, f) < 0) exit(1);
		fclose(f);
		return 0;
	}

	message(usage);
	exit(1);
}
